# -*- coding: utf-8 -*-
# Docbook XML markup check

from xml.parsers.expat import ParserCreate
from xml.parsers.expat import ExpatError
from KPC.classes import Error, BaseCheck

# 메시지에 대해 XML 파서를 돌려 well formed인지 검사하고, 태그가 알려진
# DocBook 태그인지 검사한다.

# DocBook: The Definitive Guide 기준
# http://www.docbook.org/tdg/en/html/part2.html

known_db_tags = set([
    'abbrev',
    'abstract',
    'accel',
    'ackno',
    'acronym',
    'action',
    'address',
    'affiliation',
    'alt',
    'anchor',
    'answer',
    'appendix',
    'appendixinfo',
    'application',
    'area',
    'areaset',
    'areaspec',
    'arg',
    'article',
    'articleinfo',
    'artpagenums',
    'attribution',
    'audiodata',
    'audioobject',
    'author',
    'authorblurb',
    'authorgroup',
    'authorinitials',
    'beginpage',
    'bibliocoverage',
    'bibliodiv',
    'biblioentry',
    'bibliography',
    'bibliographyinfo',
    'biblioid',
    'bibliolist',
    'bibliomisc',
    'bibliomixed',
    'bibliomset',
    'biblioref',
    'bibliorelation',
    'biblioset',
    'bibliosource',
    'blockinfo',
    'blockquote',
    'book',
    'bookinfo',
    'bridgehead',
    'callout',
    'calloutlist',
    'caption',
    'caution',
    'chapter',
    'chapterinfo',
    'citation',
    'citebiblioid',
    'citerefentry',
    'citetitle',
    'city',
    'classname',
    'classsynopsis',
    'classsynopsisinfo',
    'cmdsynopsis',
    'co',
    'code',
    'col',
    'colgroup',
    'collab',
    'collabname',
    'colophon',
    'colspec',
    'command',
    'computeroutput',
    'confdates',
    'confgroup',
    'confnum',
    'confsponsor',
    'conftitle',
    'constant',
    'constraint',
    'constraintdef',
    'constructorsynopsis',
    'contractnum',
    'contractsponsor',
    'contrib',
    'copyright',
    'coref',
    'corpauthor',
    'corpcredit',
    'corpname',
    'country',
    'database',
    'date',
    'dedication',
    'destructorsynopsis',
    'edition',
    'editor',
    'email',
    'emphasis',
    'entry',
    'entrytbl',
    'envar',
    'epigraph',
    'equation',
    'errorcode',
    'errorname',
    'errortext',
    'errortype',
    'example',
    'exceptionname',
    'fax',
    'fieldsynopsis',
    'figure',
    'filename',
    'firstname',
    'firstterm',
    'footnote',
    'footnoteref',
    'foreignphrase',
    'formalpara',
    'funcdef',
    'funcparams',
    'funcprototype',
    'funcsynopsis',
    'funcsynopsisinfo',
    'function',
    'glossary',
    'glossaryinfo',
    'glossdef',
    'glossdiv',
    'glossentry',
    'glosslist',
    'glosssee',
    'glossseealso',
    'glossterm',
    'graphic',
    'graphicco',
    'group',
    'guibutton',
    'guiicon',
    'guilabel',
    'guimenu',
    'guimenuitem',
    'guisubmenu',
    'hardware',
    'highlights',
    'holder',
    'honorific',
    'html:form',
    'imagedata',
    'imageobject',
    'imageobjectco',
    'important',
    'index',
    'indexdiv',
    'indexentry',
    'indexinfo',
    'indexterm',
    'informalequation',
    'informalexample',
    'informalfigure',
    'informaltable',
    'initializer',
    'inlineequation',
    'inlinegraphic',
    'inlinemediaobject',
    'interface',
    'interfacename',
    'invpartnumber',
    'isbn',
    'issn',
    'issuenum',
    'itemizedlist',
    'itermset',
    'jobtitle',
    'keycap',
    'keycode',
    'keycombo',
    'keysym',
    'keyword',
    'keywordset',
    'label',
    'legalnotice',
    'lhs',
    'lineage',
    'lineannotation',
    'link',
    'listitem',
    'literal',
    'literallayout',
    'lot',
    'lotentry',
    'manvolnum',
    'markup',
    'mathphrase',
    'medialabel',
    'mediaobject',
    'mediaobjectco',
    'member',
    'menuchoice',
    'methodname',
    'methodparam',
    'methodsynopsis',
    'mml:math',
    'modespec',
    'modifier',
    'mousebutton',
    'msg',
    'msgaud',
    'msgentry',
    'msgexplan',
    'msginfo',
    'msglevel',
    'msgmain',
    'msgorig',
    'msgrel',
    'msgset',
    'msgsub',
    'msgtext',
    'nonterminal',
    'note',
    'objectinfo',
    'olink',
    'ooclass',
    'ooexception',
    'oointerface',
    'option',
    'optional',
    'orderedlist',
    'orgdiv',
    'orgname',
    'otheraddr',
    'othercredit',
    'othername',
    'package',
    'pagenums',
    'para',
    'paramdef',
    'parameter',
    'part',
    'partinfo',
    'partintro',
    'personblurb',
    'personname',
    'phone',
    'phrase',
    'pob',
    'postcode',
    'preface',
    'prefaceinfo',
    'primary',
    'primaryie',
    'printhistory',
    'procedure',
    'production',
    'productionrecap',
    'productionset',
    'productname',
    'productnumber',
    'programlisting',
    'programlistingco',
    'prompt',
    'property',
    'pubdate',
    'publisher',
    'publishername',
    'pubsnumber',
    'qandadiv',
    'qandaentry',
    'qandaset',
    'question',
    'quote',
    'refclass',
    'refdescriptor',
    'refentry',
    'refentryinfo',
    'refentrytitle',
    'reference',
    'referenceinfo',
    'refmeta',
    'refmiscinfo',
    'refname',
    'refnamediv',
    'refpurpose',
    'refsect1',
    'refsect1info',
    'refsect2',
    'refsect2info',
    'refsect3',
    'refsect3info',
    'refsection',
    'refsectioninfo',
    'refsynopsisdiv',
    'refsynopsisdivinfo',
    'releaseinfo',
    'remark',
    'replaceable',
    'returnvalue',
    'revdescription',
    'revhistory',
    'revision',
    'revnumber',
    'revremark',
    'rhs',
    'row',
    'sbr',
    'screen',
    'screenco',
    'screeninfo',
    'screenshot',
    'secondary',
    'secondaryie',
    'sect1',
    'sect1info',
    'sect2',
    'sect2info',
    'sect3',
    'sect3info',
    'sect4',
    'sect4info',
    'sect5',
    'sect5info',
    'section',
    'sectioninfo',
    'see',
    'seealso',
    'seealsoie',
    'seeie',
    'seg',
    'seglistitem',
    'segmentedlist',
    'segtitle',
    'seriesvolnums',
    'set',
    'setindex',
    'setindexinfo',
    'setinfo',
    'sgmltag',
    'shortaffil',
    'shortcut',
    'sidebar',
    'sidebarinfo',
    'simpara',
    'simplelist',
    'simplemsgentry',
    'simplesect',
    'spanspec',
    'state',
    'step',
    'stepalternatives',
    'street',
    'structfield',
    'structname',
    'subject',
    'subjectset',
    'subjectterm',
    'subscript',
    'substeps',
    'subtitle',
    'superscript',
    'surname',
    'svg:svg',
    'symbol',
    'synopfragment',
    'synopfragmentref',
    'synopsis',
    'systemitem',
    'table',
    'task',
    'taskprerequisites',
    'taskrelated',
    'tasksummary',
    'tbody',
    'td',
    'term',
    'termdef',
    'tertiary',
    'tertiaryie',
    'textdata',
    'textobject',
    'tfoot',
    'tgroup',
    'th',
    'thead',
    'tip',
    'title',
    'titleabbrev',
    'toc',
    'tocback',
    'tocchap',
    'tocentry',
    'tocfront',
    'toclevel1',
    'toclevel2',
    'toclevel3',
    'toclevel4',
    'toclevel5',
    'tocpart',
    'token',
    'tr',
    'trademark',
    'type',
    'ulink',
    'uri',
    'userinput',
    'varargs',
    'variablelist',
    'varlistentry',
    'varname',
    'videodata',
    'videoobject',
    'void',
    'volumenum',
    'warning',
    'wordasword',
    'xref',
    'year'
])

tag_error_string = 'XML 태그의 짝이 맞지 않습니다'
notdb_error_string = '<%s>: 알려진 DocBook 태그가 아닙니다'


class NotDocBook(Exception):
    pass


def check_db_tags(name):
    if name == 'KPC_DummyTag':
        pass
    elif name[:12] == 'placeholder-':
        # old gnome-doc-utils magic
        pass
    elif name[:7] == '_:item-' or name[:7] == '_:link-':
        # newer gnome-doc-utils magic
        pass
    elif name not in known_db_tags:
        raise NotDocBook(name)


def start_element(name, attr):
    check_db_tags(name)


def end_element(name):
    check_db_tags(name)


class MarkupDocbookCheck(BaseCheck):
    def check(self, entry, context):
        if not entry.references or '.xml:' not in entry.references[0]:
            # not from an XML file
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        if msgid == 'translator-credits':  # gnome-doc-utils magic
            return []
        parser = ParserCreate()
        parser.StartElementHandler = start_element
        parser.EndElementHandler = end_element
        parser.UseForeignDTD(True)
        try:
            parser.Parse('<KPC_DummyTag>' + msgstr + '</KPC_DummyTag>')
        except ExpatError as e:
            return [Error(tag_error_string)]
        except NotDocBook as e:
            return [Error(notdb_error_string % e)]
        return []

name = 'convention/markup-docbook'
description = 'DocBook 마크업의 짝이 맞는지 검사합니다'
checker = MarkupDocbookCheck()



# -*- coding: utf-8 -*-

import re
from KPC.classes import Error, BaseCheck

typo_re = re.compile('([\u3131-\u318E]+)')
# 예외: 인덱스 표시 ('ㄱ-ㅅ' 등)
typo_exception_re = re.compile('^[\u3131-\u318E]-[\u3131-\u318E]')
typo_error = '\"%s\": 음절이 불완전합니다.  오타로 보입니다'


class TypoIncompleteCheck(BaseCheck):
    def check(self, entry, context):
        msgstr = entry.msgstr
        errors = []
        s = msgstr
        while 1:
            mo = typo_re.search(s)
            if mo and not typo_exception_re.match(s):
                errors.append(Error(typo_error % mo.group(1)))
                s = s[mo.end():]
            else:
                break
        return errors

name = 'language/typo-incomplete'
description = '오타로 보이는 불완전한 음절을 찾아냅니다'
checker = TypoIncompleteCheck()



class Error:
    '''An individual error from a check against an entry. One check
    usually returns list of errors or empty list if there's no error.
    '''
    SUCCESS = 0
    DEPRECATED = 1
    WARNING = 2
    ERROR = 3

    def __init__(self, message):
        self.pos = 0
        self.level = Error.ERROR
        self.message = message


class BaseCheck:
    '''Base class to check an entry from PO files. Subclasses should
    implement check() function. check() returns list of errors if any.
    '''
    def _get_context(self, entry):
        if not hasattr(self, 'get_context'):
            return {}
        fields = {}
        # parse msgstr
        for line in entry.msgstr.split('\n'):
            try:
                k, v = line.split(': ', 1)
                fields[k] = v
            except ValueError:
                pass
        return self.get_context(entry, fields)

    def _check(self, entry, context):
        return self.check(entry, context)


class HeaderCheck(BaseCheck):
    '''Base class to check the header entries. Subclasses should
    implement check_header() function.
    '''
    def check(self, entry, context):
        # header messages only
        if entry.msgid != '':
            return []
        fields = {}
        # parse msgstr
        for line in entry.msgstr.split('\n'):
            try:
                k, v = line.split(': ', 1)
                fields[k] = v
            except ValueError:
                pass
        return self.check_header(entry, fields)


class CheckList(BaseCheck):
    '''Container class to perform all child checks
    '''
    def __init__(self, children=[]):
        self.children = children.copy()

    def check(self, entry, context):
        r = []
        for c in self.children:
            k = c.check(entry, context)
            if k:
                r += k
        return r



# -*- coding: utf-8 -*-

from KPC.classes import Error, HeaderCheck, CheckList


class NoPoliticsCheck(HeaderCheck):
    keywords = ['dokdo']
    errmsg = '정치적 문구를 쓰는 곳이 아닙니다: %s'

    def check_header(self, entry, fields):
        comment_lines = entry.translator_comment.split('\n')
        r = []
        for k in self.keywords:
            try:
                if k in fields['Last-Translator'].lower():
                    r.append(Error(self.errmsg % fields['Last-Translator']))
                if k in fields['Language-Team'].lower():
                    r.append(Error(self.errmsg % fields['Language-Team']))
                for l in comment_lines:
                    if k in l.lower():
                        r.append(Error(self.errmsg % l))
            except KeyError:
                pass
        return r


name = 'convention/team-common'
description = '번역팀 공통 관련 사항을 검사합니다'
checker = CheckList([NoPoliticsCheck()])



# -*- coding: utf-8 -*-
# extra space

from KPC.classes import Error, BaseCheck


class ExtraSpaceCheck(BaseCheck):
    def check(self, entry, context):
        msgid = entry.msgid
        msgstr = entry.msgstr

        if msgstr[-1].isspace() and msgid != '' and msgstr[-1] != msgid[-1]:
            return [Error('끝에 불필요하게 공백 문자가 있습니다')]
        return []

name = 'convention/extra-space'
description = '불필요한 공백 문자 검사'
checker = ExtraSpaceCheck()



# -*- coding: utf-8 -*-

# 그놈 프로그램의 도움말 문서의 법률 정보 페이지에 공통으로 많이
# 사용하는 Creative Commons 라이선스 안내 문구 번역 통일

import re
from KPC.classes import Error, BaseCheck

data = [
    {'msgid': 'Legal information\.',
     'msgstr': '법적 정보.'},
    {'msgid': 'As a special exception, the copyright holders give you ' +
              'permission to copy, modify, and distribute the example code ' +
              'contained  in this document under the terms of your ' +
              'choosing, without restriction.',
     'msgstr': '예외적으로, 이 문서에 들어 있는 예제 코드는 어떤 조건이든 제한 ' +
               '없이 복사, 수정, 배포할 수 있도록 저작권자가 특별히 허용합니다.'},
    {'msgid': 'This work is distributed under a CreativeCommons ' +
              'Attribution-Share Alike 3\.0 Unported license\.',
     'msgstr': '이 문서는 크리에이티브 커먼즈 저작자표시-동일조건변경허락 ' +
               '3.0 Unported 라이선스로 배포됩니다.'},
    {'msgid': 'Creative ?Commons Share ?Alike 3\.0',
     'msgstr': '크리에이티브 커먼즈 동일조건변경허락 3.0'},
    {'msgid': 'Creative ?Commons Share ?Alike 3\.0 United States License',
     'msgstr': '크리에이티브 커먼즈 동일조건변경허락 3.0 미합중국 라이선스'},
    {'msgid': 'Creative ?Commons Attribution-Share ?Alike 3\.0 United ' +
              'States License',
     'msgstr': '크리에이티브 커먼즈 저작자표시-동일조건변경허락 3.0 미합중국 라이선스'},
    {'msgid': 'Creative ?Commons Attribution-Share ?Alike 3\.0 Unported ' +
              'License',
     'msgstr': '크리에이티브 커먼즈 저작자표시-동일조건변경허락 3.0 Unported 라이선스'},
    {'msgid': '<em>To share</em>',
     'msgstr': '<em>공유</em>'},
    {'msgid': '<em>To remix</em>',
     'msgstr': '<em>변경</em>'},
    {'msgid': '<em>Attribution</em>',
     'msgstr': '<em>저작자 표시</em>'},
    {'msgid': '<em>Share Alike</em>',
     'msgstr': '<em>동일 조건</em>'},
]


class HelpLicenseCheck(BaseCheck):
    errstr = '일관성을 위해 다음과 같이 번역합니다: %s'

    def check(self, entry, context):
        for p in entry.references:
            if '.page' in p or '.xml' in p:
                break
        else:
             return []
        for d in data:
            if not re.match(d['msgid'], entry.msgid, re.IGNORECASE):
                continue
            if entry.msgstr != d['msgstr']:
                return [Error(self.errstr % d['msgstr'])]
        return []

name = 'terminology/help-license'
description = '문서의 Creative Commons 라이선스 관련 번역이 통일된 번역인지 검사합니다'
checker = HelpLicenseCheck()



# -*- coding: utf-8 -*-

import re
from KPC.classes import Error, BaseCheck

check_list = [
    {'check': lambda e: True not in [('.desktop' in p) for p in e.references],
     're': re.compile('\s([A-Za-z\uac00-\ud7a3]*[\uac00-\ud7a3];)\s+',
                      re.UNICODE | re.MULTILINE),
     'msg': '우리말 맞춤법에서는 세미콜론을 쓰지 않습니다'},
]


class PunctuationCheck(BaseCheck):
    def check(self, entry, context):
        msgstr = entry.msgstr
        errors = []
        for i in check_list:
            if 'check' in i and not i['check'](entry):
                continue
            mo = i['re'].search(msgstr)
            if mo:
                errors.append(Error(i['msg']))
        return errors

name = 'language/punctuation'
description = '잘못된 한국어 구문 규칙을 찾아냅니다'
checker = PunctuationCheck()
